//
// GameX
// Ship Demo - Main Program
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

#define GAMEX_MAIN
#pragma comment (lib, "gamex.lib")
#include "gamex.hpp"

#include "shipdemox-ship.hpp"				// Include for Ship Class
#include "shipdemox-photons.hpp"			// Include for Photons Class

											//*** Global Variables
Ship ship_blue, ship_red;					// Red and Blue ship objects
Photons photons;							// Photon object
ImageX back_img;							// Background Image

// Required GameX initialize function (called when program starts)
// this function should initialize GameX and prepare anything else we need
void GameInit (void)
{											// Initialize GameX itself (required)
	GameX.Initialize ("ShipDemoX",VIDEO_16BIT|VIDEO_ALLOWREFRESHSYNC, 800, 400);

	photons.Initialize ();					// Initialize Photons
	ship_red.Initialize (1);				// Initialize Red Ship  (controlled by WASDX keys)
	ship_blue.Initialize (0);				// Initialize Blue Ship (controlled by arrow keys + space)
	back_img.Load ("Images\\ice.jpg");		// Load the background
}

// Required GameX main function (called repeatedly while game is running)
// this function should update the game state but should NOT draw that state
void GameRun (void)
{											// Update Ships & Photons
	ship_blue.Update ();					// (call respective class member functions)
	ship_red.Update ();
	photons.Update ();	

	ship_red.CollideCheck (&ship_blue);		// Ship-to-Ship Collision Detection
}

// Required GameX draw function (called repeatedly while game is running)
// this function should draw the game state and do as little else as possible
void GameDraw (void)
{										// Draw Background (ice), offset opposite of ships
	GameX.DrawTiled (&back_img, ship_red.GetX()+ship_blue.GetX(), ship_red.GetY()+ship_blue.GetY());
	ship_blue.Draw ();					// Draw Red and Blue ships
	ship_red.Draw ();		
	photons.Draw ();					// Draw Photons
}
