//
// GameX
// Text Printer Class Header
// by Justin Pease
// 
// This software is released under the GameX GNU GPL 
// Open Source License. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 


#ifndef DP_TEXT
	#define DP_TEXT

	#include "gamex.hpp"
							
	class TextPrinter {
	public:
		// Initialize text printer (by loading text graphics)
		// Just call tp.Initialize("Images\\text.bmp"); once inside GameInit()
		// and everything that includes "text.h" will be able to call tp.PrintText().
		void Initialize (char * alphabet_image_path);

		// Prints out a string of text graphically using text_img which was loaded from bmp
		// Believe it or not, this is faster than using DrawText() to draw text, and much more flexible.
		// - font_size is the point size of the font (dimensions in pixels of each character)
		// - draw_mode is a GameX draw mode such as DRAW_GHOST or DRAW_ADD or DRAW_SUBTRACT|DRAWOP_NOFILTER etc.
		// - if rotate_baseline is false, angle will rotate all the individual letters instead of the whole string
		// - if fixed is true, the text is printed like a fixed-width font (maybe too wide, should fix)
		void PrintText (int draw_mode, float font_size, char * str, int x, int y, int r,int g,int b,int a, int r2,int g2,int b2,int a2, float angle=0.0f, float scaley=1.0f, bool rotate_baseline=true, bool fixed = false);
		inline void PrintText (int draw_mode, float font_size, char * str, int x, int y, int r=255,int g=255,int b=255,int a=255, float angle=0.0f, float scaley=1.0f, bool rotate_baseline=true, bool fixed = false) {PrintText(draw_mode,font_size,str,x,y,r,g,b,a,r,g,b,a,angle,scaley,rotate_baseline,fixed);}
		float GetVisibleCharWidth(char * str, int index, float font_size);
		void GetVisibleCharInfo(char * str, int n, int & xs, int & ys, float & kern, bool & visible);
		float GetVisibleStringWidth(char * str, int first_index, int last_index, float font_size);
		float GetVisibleStringWidth(char * str, float font_size);
	private:
		ImageX text_img;
	};

	#ifdef GAMEX_MAIN // act sort of like an extension of GameX
		TextPrinter tp;
	#else
		extern TextPrinter tp; // where tp is a global TextPrinter object
	#endif

#endif
